//-----------------------------------------------------------------------------
// Title      : bsp_ub057_mk3_ir_rc5
//-----------------------------------------------------------------------------
// Funktion   : Infrarot-Daten auslesen und anzeigen (RC5)
// Schaltung  : myAVR MK3
//-----------------------------------------------------------------------------
// Prozessor  : ATMega2560
// Takt       : 16 MHz
// Sprache    : C
// Date       : 09.11.2012
// Version    : 1.0
// Autor      : UB
//-----------------------------------------------------------------------------
#define F_CPU 16000000
#include <avr/io.h>
#include <ub_mk3_2560/MK3_2560_LCD.h>
#include <ub_mk3_2560/MK3_2560_IR_RC5.h>

//-----------------------------------------------------------------------------
// Hinweis : Quick-Jumper Port-C+A muss gesetzt sein
// - ein IR-Sensor (z.B. SFH-5110) muss an Port-L Bit-4 angeschlossen sein
// - eine Fernbedienung mit Code-Typ : RC5 muss benutzt werden
//-----------------------------------------------------------------------------


int main (void)
{
	IRRC_Status_t status;
	unsigned int n;
	unsigned char m=0;

	MK3_INIT_LCD();					// init des LCD

	MK3_LCD_CLEAR();				// LCD loeschen
	MK3_LCD_LIGHT(true);			// beleuchtung einschalten
	
	MK3_LCD_STRING_AT(0,0,"IR-Test [Code:RC5]");

	// IR-Funktion initialisieren
	MK3_INIT_IRRC();

	while (true)
	{
		// IR-Status abfragen
		status=MK3_IRRC_check();
		if(status!=IRRC_NO_CMD) {
		  // wenn ein IR-Commando empfangen wurde
		  
		  if(status==IRRC_CMD_OK)     MK3_LCD_STRING_AT(0,16,"RX=OK    ");
		  if(status==IRRC_CMD_REPEAT) MK3_LCD_STRING_AT(0,16,"RX=Repeat");
		  if(status==IRRC_CMD_ERR)    MK3_LCD_STRING_AT(0,16,"RX=Error ");
		  if(status==IRRC_TYP_ERR)    MK3_LCD_STRING_AT(0,16,"TYP=Error");
		  
		  // anzeigen der empfangenen Bytes
		  MK3_LCD_STRING_AT(0,24,"Data=");
		  for(n=0;n<IRRC_DATA_BYTES;n++) {
		  	MK3_LCD_PRINT_UTYP(IRRC.ir_code[n],HEX_U2);
		  	MK3_LCD_PRINT_STRING(",");
		  }
		  // Typ anzeigen
		  if(IRRC.ir_typ==IRRC_SIRCS)     MK3_LCD_STRING_AT(0,32,"Typ=SIRCS"); 
		  if(IRRC.ir_typ==IRRC_RC5)       MK3_LCD_STRING_AT(0,32,"Typ=RC5  ");
		  if(IRRC.ir_typ==IRRC_UNDEFINED) MK3_LCD_STRING_AT(0,32,"Typ=?    ");
		  
		  // Symbol anzeigen
		  m++;
		  if(m>3) m=0;
		  if(m==0) MK3_LCD_STRING_AT(0,48,".   ");
		  if(m==1) MK3_LCD_STRING_AT(0,48,"..  ");
		  if(m==2) MK3_LCD_STRING_AT(0,48,"... ");
		  if(m==3) MK3_LCD_STRING_AT(0,48,"....");
		}
	}
	return 0;
}
//-----------------------------------------------------------------------------






